@@ -7,9 +7,11 @@ use warnings;
 use File::ShareDir;
 use File::Spec;
 
+use Log::Handler;
+
 use Moo;
 
-use Types::Standard qw/Int Str/;
+use Types::Standard qw/Any Int Str/;
 
 has config_file =>
 (
@@ -21,7 +23,31 @@ has config_file =>
 
 has data_file =>
 (
-	default  => sub{return 'data/en.wikipedia.org.wiki.ISO_3166-2'},
+	default  => sub{return 'data/en.wikipedia.org.wiki.ISO_3166-1'},
+	is       => 'rw',
+	isa      => Str,
+	required => 0,
+);
+
+has logger =>
+(
+	default  => sub{return undef},
+	is       => 'rw',
+	isa      => Any,
+	required => 0,
+);
+
+has maxlevel =>
+(
+	default  => sub{return 'notice'},
+	is       => 'rw',
+	isa      => Str,
+	required => 0,
+);
+
+has minlevel =>
+(
+	default  => sub{return 'error'},
 	is       => 'rw',
 	isa      => Str,
 	required => 0,
@@ -43,23 +69,30 @@ has sqlite_file =>
 	required => 0,
 );
 
-has verbose =>
-(
-	default  => sub{return 0},
-	is       => 'rw',
-	isa      => Int,
-	required => 0,
-);
-
-our $VERSION = '1.04';
+our $VERSION = '2.00';
 
 # -----------------------------------------------
 
 sub BUILD
 {
-	my($self, $arg)    = @_;
-	(my $package       = __PACKAGE__) =~ s/::/-/g;
-	my($dir_name)      = $ENV{AUTHOR_TESTING} ? 'share' : File::ShareDir::dist_dir($package);
+	my($self)		= @_;
+	(my $package	= __PACKAGE__) =~ s/::/-/g;
+	my($dir_name)	= $ENV{AUTHOR_TESTING} ? 'share' : File::ShareDir::dist_dir($package);
+
+	if (! defined $self -> logger)
+	{
+		$self -> logger(Log::Handler -> new);
+		$self -> logger -> add
+		(
+			screen =>
+			{
+				maxlevel       => $self -> maxlevel,
+				message_layout => '%m',
+				minlevel       => $self -> minlevel,
+				utf8           => 1,
+			}
+		);
+	}
 
 	$self -> config_file(File::Spec -> catfile($dir_name, $self -> config_file) );
 	$self -> sqlite_file(File::Spec -> catfile($dir_name, $self -> sqlite_file) );
@@ -71,27 +104,15 @@ sub BUILD
 sub log
 {
 	my($self, $level, $s) = @_;
-	$level ||= 'debug';
-	$s     ||= '';
+	$level = 'notice' if (! defined $level);
+	$s     = ''       if (! defined $s);
 
-	print "$level: $s. \n" if ($self -> verbose);
+	$self -> logger -> $level($s) if ($self -> logger);
 
 }	# End of log.
 
 # -----------------------------------------------
 
-sub run
-{
-	my($self) = @_;
-
-	# Return 0 for success and 1 for failure.
-
-	return 0;
-
-} # End of run.
-
-# -----------------------------------------------
-
 1;
 
 =pod
@@ -106,7 +127,9 @@ WWW::Scraper::Wikipedia::ISO3166 - Gently scrape Wikipedia for ISO3166-2 data
 
 Wikipedia I<has been scraped>. You do not need to run the scripts which download pages from there.
 
-Just use the SQLite database shipped with this module, as discussed next.
+Just use the SQLite database shipped with this module, share/www.scraper.wikipedia.iso3166.sqlite.
+
+See scripts/export.*.pl and scripts/get.*.pl for sample code.
 
 =head2 Methods which return hashrefs
 
@@ -115,9 +138,11 @@ Just use the SQLite database shipped with this module, as discussed next.
 	my($database)     = WWW::Scraper::Wikipedia::ISO3166::Database -> new;
 	my($countries)    = $database -> read_countries_table;
 	my($subcountries) = $database -> read_subcountries_table;
+	my($categories)   = $database -> read_subcountry_categories_table;
+	my($types)        = $database -> read_subcountry_info_table;
 	...
 
-Each key in %$countries and %$subcountries points to a hashref of all columns for the given key.
+Each key in returned C<hashrefs> points to a hashref of all columns for the given key.
 
 So, $$countries{13} points to this hashref:
 
@@ -128,50 +153,54 @@ So, $$countries{13} points to this hashref:
 		fc_name           => 'australia',
 		hash_subcountries => 'Yes',
 		name              => 'Australia',
+		number            => '036',
 		timestamp         => '2012-05-08 04:04:43',
 	}
 
-One element of %$subcountries is $$subcountries{4276}:
+One element of %$subcountries is $$subcountries{941}:
 
 	{
-		id         => 4276,
-		country_id => 13,
-		code       => 'AU-VIC',
-		fc_name    => 'victoria',
-		name       => 'Victoria',
-		sequence   => 5,
-		timestamp  => '2012-05-08 04:05:27',
+		id                     => 941,
+		country_id             => 13,
+		code                   => 'AU-VIC',
+		fc_name                => 'victoria',
+		name                   => 'Victoria',
+		sequence               => 7,
+		subcountry_category_id => 8,
+		timestamp              => '2012-05-08 04:05:27',
 	}
 
 =head3 Warnings
 
-# 1: These hashrefs use the table's primary key as the hashref's key. In the case of the I<countries>
+These hashrefs use the table's primary key as the hashref's key. In the case of the I<countries>
 table, the primary key is the country's id, and is used as subcountries.country_id. But, in the case of
 the I<subcountries> table, the id does not have any meaning apart from being a db primary key.
 See L</What is the database schema?> for details.
 
-# 2: Do not assume subcountry names are unique within a country.
-
-L<See 'Taichung' etc in Taiwan for example|http://en.wikipedia.org/wiki/ISO_3166-2:TW>.
-
 =head2 Scripts which output to a file
 
-All scripts respond to the -h option.
+Note: Many of these programs respond to the -h command line switch, but not create.tables.pl nor
+drop.tables.pl.
 
 Some examples:
 
-	shell>perl scripts/export.as.csv.pl -c countries.csv -s subcountries.csv
-	shell>perl scripts/export.as.html.pl -w iso.3166-2.html
+	shell> perl scripts/export.as.csv.pl -c countries.csv -s subcountries.csv
+	shell> perl scripts/export.as.html.pl -w iso.3166-2.html
+	shell> perl -Ilib scripts/populate.countries.pl -maxlevel debug
+	shell> perl -Ilib scripts/populate.subcountries.pl -maxlevel debug
 
-This file is on-line at: L<http://savage.net.au/Perl-modules/html/WWW/Scraper/Wikipedia/ISO3166/iso.3166-2.html>.
+The HTML file is on-line at: L<http://savage.net.au/Perl-modules/html/WWW/Scraper/Wikipedia/ISO3166/iso.3166-2.html>.
 
 	shell>perl scripts/report.statistics.pl
 
 	Output statistics:
-	countries_in_db => 249.
-	has_subcounties => 199.
-	subcountries_in_db => 4593.
-	subcountry_files_downloaded => 249.
+	countries_in_db => 249
+	has_subcounties => 200
+	subcountries_in_db => 5297
+	subcountry_files_downloaded => 249
+	subcountry_info_in_db => 352
+
+See also scripts/report.*.pl and t/report.t.
 
 =head1 Description
 
@@ -186,33 +215,78 @@ Likewise, the data has been imported.
 
 This means you would normally only ever use the database in read-only mode.
 
-Its components are:
+Note: Many of these programs respond to the -h command line switch, but not create.tables.pl nor
+drop.tables.pl.
+
+Scripts, all shipped in scripts/:
 
 =over 4
 
-=item o scripts/get.country.page.pl
+=item o build.database.sh
 
-1: Downloads the ISO3166-1_alpha-3 page from Wikipedia.
+Mainly for use by me. It runs:
 
-Input: L<http://en.wikipedia.org/wiki/ISO_3166-1_alpha-3>.
+=over 4
 
-Output: data/en.wikipedia.org.wiki.ISO_3166-2.3.html.
+=item o perl -Ilib scripts/drop.tables.pl
 
-2: Downloads the ISO3166-2 page from Wikipedia.
+=item o perl -Ilib scripts/create.tables.pl
 
-Input: L<http://en.wikipedia.org/wiki/ISO_3166-2>.
+=item o perl -Ilib scripts/populate.countries.pl -maxlevel debug
 
-Output: data/en.wikipedia.org.wiki.ISO_3166-2.html.
+=item o perl -Ilib scripts/populate.subcountries.pl -maxlevel debug
 
-=item o scripts/populate.countries.pl
+=item o perl -Ilib scripts/export.as.html.pl -w data/iso.3166-2.html
 
-Imports country data into an SQLite database.
+=item o cp data/iso.3166-2.html $DR/
 
-inputs: data/en.wikipedia.org.wiki.ISO_3166-2.html, data/en.wikipedia.org.wiki.ISO_3166-2.3.html.
+$DR is my web site's RAMdisk-based doc root.
 
-Output: share/www.scraper.wikipedia.iso3166.sqlite.
+=item perl -Ilib scripts/export.as.csv.pl \
 
-=item o scripts/get.subcountry.page.pl and scripts/get.subcountry.pages.pl
+=back
+
+=item o check.downloads.pl
+
+Ensure each subcountry file has been downloaded, and report any which haven't been. Also report
+and unexpected subcountry files found in data/.
+
+=item o perl -Ilib scripts/create.tables.pl
+
+=item o perl -Ilib scripts/drop.tables.pl
+
+=item o export.as.csv.pl -country_file c.csv -subcountry_file s.csv subcountry_info_file i.csv
+
+Exports the country, subcountry and subcountry info data as CSV.
+
+Input: share/www.scraper.wikipedia.iso3166.sqlite.
+
+Output: data/countries.csv and data/subcountries.csv.
+
+=item o export.as.html -w c.html
+
+Exports the country and subcountry data as HTML.
+
+Input: share/www.scraper.wikipedia.iso3166.sqlite.
+
+Output: data/iso.3166-2.html.
+
+On-line: L<http://savage.net.au/Perl-modules/html/WWW/Scraper/Wikipedia/ISO3166/iso.3166-2.html>.
+
+=item o find.db.pl
+
+After installation, this will print the path to www.scraper.wikipedia.iso3166.sqlite.
+
+=item o get.country.pages.pl
+
+1: Downloads the ISO3166-1 and ISO3166-2 pages from Wikipedia.
+
+Input: L<https://en.wikipedia.org/wiki/ISO_3166-1> and
+<https://en.wikipedia.org/wiki/ISO_3166-2>.
+
+Output: data/en.wikipedia.org.wiki.ISO_3166-1.html and data/en.wikipedia.org.wiki.ISO_3166-2.html.
+
+=item o get.subcountry.page.pl and scripts/get.subcountry.pages.pl
 
 Downloads each countries' corresponding subcountries page.
 
@@ -220,7 +294,20 @@ Source: http://en.wikipedia.org/wiki/ISO_3166:$code2.html.
 
 Output: data/en.wikipedia.org.wiki.ISO_3166-2.$code2.html.
 
-=item o scripts/populate.subcountry.pl and scripts/populate.subcountries.pl
+=item o pod2html.sh
+
+For use by the author. It converts each *.pm file into the corresponding *.html file, and outputs
+them to my web server's doc root.
+
+=item o populate.countries.pl
+
+Imports country data into an SQLite database.
+
+Input: data/en.wikipedia.org.wiki.ISO_3166-1.html, data/en.wikipedia.org.wiki.ISO_3166-2.html.
+
+Output: share/www.scraper.wikipedia.iso3166.sqlite.
+
+=item o populate.subcountry.pl and scripts/populate.subcountries.pl
 
 Imports subcountry data into the database.
 
@@ -231,38 +318,17 @@ Output: share/www.scraper.wikipedia.iso3166.sqlite.
 Note: When the distro is installed, this SQLite file is installed too.
 See L</Where is the database?> for details.
 
-=item o scripts/export.as.csv.pl -c c.csv -s s.csv
+=item o report.Australian.statistics.pl
 
-Exports the country and subcountry data as CSV.
+A simple test program. See also the next script.
 
-Input: share/www.scraper.wikipedia.iso3166.sqlite.
+Run it with the '-max info' command line options.
 
-Output: data/countries.csv and data/subcountries.csv.
+=item o report.statistics.pl
 
-=item o scripts/export.as.html -w c.html
+A simple test program. See also the previous script.
 
-Exports the country and subcountry data as HTML.
-
-Input: share/www.scraper.wikipedia.iso3166.sqlite.
-
-Output: data/iso.3166-2.html.
-
-On-line: L<http://savage.net.au/Perl-modules/html/WWW/Scraper/Wikipedia/ISO3166/iso.3166-2.html>.
-
-=item o scripts/get.statoids.pl
-
-Downloads some pages from L<http://statoids.com> in case one day we need to convert from FIPS to ISO 3166-2.
-
-See data/List_of_FIPS_region_codes_*.html.
-
-=item o scripts/populate.fips.codes.pl
-
-This reads the files output by scripts/get.statoids.pl and produces 2 reports, data/wikipedia.fips.codes.txt
-and data/wikipedia.fips.mismatch.log. These are discussed in L</What FIPS data is included?>
-
-=item o scripts/test.nfc.pl
-
-See L</Why did you use C<Unicode::Normalize>'s NFC() for sorting?> for a discussion of this script.
+Run it with the '-max info' command line options.
 
 =back
 
@@ -291,6 +357,60 @@ The code prefixes this name with the directory returned by L<File::ShareDir/dist
 
 Default: .htwww.scraper.wikipedia.iso3166.conf.
 
+=item o logger => $aLoggerObject
+
+Specify a logger compatible with L<Log::Handler>, for the lexer and parser to use.
+
+Default: A logger of type L<Log::Handler> which writes to the screen.
+
+To disable logging, just set 'logger' to the empty string (not undef).
+
+=item o maxlevel => $logOption1
+
+This option affects L<Log::Handler>.
+
+Possible values for C<maxlevel> and C<minlevel> are:
+
+=over 4
+
+=item o debug
+
+Generates the maximum amount of output.
+
+=item o info
+
+=item o notice
+
+By default, C<notice> is the highest level used.
+
+=item o warning, warn
+
+=item o error, err
+
+By default, C<error> is the lowest level used.
+
+=item o critical, crit
+
+=item o alert
+
+=item o emergency, emerg
+
+=back
+
+See the L<Log::Handler::Levels> docs.
+
+Default: 'notice'.
+
+=item o minlevel => $logOption2
+
+This option affects L<Log::Handler>.
+
+See the L<Log::Handler::Levels> docs.
+
+Default: 'error'.
+
+No lower levels are used.
+
 =item o sqlite_file => $file_name
 
 The name of the SQLite database of country and subcountry data.
@@ -299,12 +419,6 @@ The code prefixes this name with the directory returned by L<File::ShareDir/dist
 
 Default: www.scraper.wikipedia.iso3166.sqlite.
 
-=item o verbose => $integer
-
-Print more or less information.
-
-Default: 0 (print nothing).
-
 =back
 
 =head1 Distributions
@@ -341,13 +455,45 @@ Get or set the name of the config file.
 
 The code prefixes this name with the directory returned by L<File::ShareDir/dist_dir()>.
 
-Also, I<config_file> is an option to L</new()>.
+C<config_file> is an option to L</new()>.
 
-=head2 log($level => $s)
+=head2 log($level, $s)
 
-Print $s at log level $level, if ($self -> verbose);
+If a logger is defined, this logs the message $s at level $level.
 
-Since $self -> verbose defaults to 0, nothing is printed by default.
+=head2 logger([$logger_object])
+
+Here, the [] indicate an optional parameter.
+
+Get or set the logger object.
+
+To disable logging, just set 'logger' to the empty string (not undef), in the call to L</new()>.
+
+This logger is passed to other modules.
+
+C<logger> is a parameter to L</new()>. See L</Constructor and Initialization> for details.
+
+=head2 maxlevel([$string])
+
+Here, the [] indicate an optional parameter.
+
+Get or set the value used by the logger object.
+
+This option is only used if an object of type L<Log::Handler> is ceated.
+See L<Log::Handler::Levels>.
+
+C<maxlevel> is a parameter to L</new()>. See L</Constructor and Initialization> for details.
+
+=head2 minlevel([$string])
+
+Here, the [] indicate an optional parameter.
+
+Get or set the value used by the logger object.
+
+This option is only used if an object of type L<Log::Handler> is created.
+See L<Log::Handler::Levels>.
+
+C<minlevel> is a parameter to L</new()>. See L</Constructor and Initialization> for details.
 
 =head2 new()
 
@@ -359,58 +505,62 @@ Get or set the name of the database file.
 
 The code prefixes this name with the directory returned by L<File::ShareDir/dist_dir()>.
 
-Also, I<sqlite_file> is an option to L</new()>.
-
-=head2 verbose($integer)
-
-Get or set the verbosity level.
-
-Also, I<verbose> is an option to L</new()>.
+C<sqlite_file> is an option to L</new()>.
 
 =head1 FAQ
 
 =head2 Design faults in ISO3166
 
-Where ISO3166 uses Country Name, I would have used Long Name and Short Name.
+Where ISO3166 uses Country Name, I would have used C<Long Name> and C<Short Name>.
 
 Then we'd have:
 
-	Long Name:  Bolivia, Plurinational State of
+	Long Name:  Bolivia (Plurinational State of)
 	Short Name: Bolivia
 
-This distro uses the value directly from Wikipedia, which is what I have called 'Long Name', for
-all country and subcountry names.
+This distro uses the values directly from Wikipedia, which is what I have called C<Long Name> above,
+for all country and subcountry names.
+
+=head2 Are any names modified by the code?
+
+Yes. &#39; is converted into a single quote.
 
 =head2 Where is the database?
 
 It is shipped in share/www.scraper.wikipedia.iso3166.sqlite.
 
 It is installed into the distro's shared dir, as returned by L<File::ShareDir/dist_dir()>.
-On my machine that's:
+Run scripts/find.db.pl to see what dir it is on your machine.
 
-/home/ron/perl5/perlbrew/perls/perl-5.14.2/lib/site_perl/5.14.2/auto/share/dist/WWW-Scraper-Wikipedia-ISO3166/www.scraper.wikipedia.iso3166.sqlite.
+On my machine it's:
+
+/home/ron/perl5/perlbrew/perls/perl-5.20.2/lib/site_perl/5.20.2/auto/share/dist/WWW-Scraper-Wikipedia-ISO3166/www.scraper.wikipedia.iso3166.sqlite
 
 =head2 What is the database schema?
 
-A single SQLite file holds 2 tables, I<countries> and I<subcountries>:
+A single SQLite file holds 4 tables:
 
-	countries           subcountries
-	---------           ------------
-	id                  id
-	code2               country_id
-	code3               code
-	fc_name             fc_name
-	has_subcountries    name
-	name                sequence
+	countries           subcountries              subcountry_categories    subcountry_info
+	---------           ------------              ---------------------    ---------------
+	id                  id                        id                       id
+	code2               country_id                name                     country_id
+	code3               subcountry_category_id    timestamp                name
+	fc_name             fc_name                                            sequence
+	has_subcountries    code                                               timestamp
+	name                name
+	number              sequence
 	timestamp           timestamp
 
-I<code3> has a couple of special cases. 2 countries have no value for code3:
-Libyan Arab Jamahiriya and Sint Maarten.
-3-letter codes which almost match: LBY => Libya and MAF => Saint Martin (French part).
+An SVG image of the schema is shipped as data/www.scraper.wikipedia.iso3166.schema.svg,
+and is L<on-line|http://savage.net.au/assets/images/modules/WWW/Scraper/Wikipedia/ISO3166/www.scraper.wikipedia.iso3166.schema.svg>.
+
+The schema of the C<countries> table is basically taken straight from the big table on
+L<ISO_3166-1|https://en.wikipedia.org/wiki/ISO_3166-1>. Likewise for the subcountry_info table,
+it's taken from L<ISO_3166-2|https://en.wikipedia.org/wiki/ISO_3166-2>.
 
 I<subcountries.country_id> points to I<countries.id>.
 
-I<fc_name> is output from calling fc(decode('utf8', $name) ).
+I<fc_name> is output from calling fc($name). It's in UTF-8.
 
 For decode(), see L<Encode/THE PERL ENCODING API>.
 
@@ -420,14 +570,30 @@ $name is from a Wikipedia page.
 
 I<has_subcountries> is 'Yes' or 'No'.
 
-I<name> is output from calling decode('utf8', $name).
+I<name> is in UTF-8.
 
-I<sequence> is a number (1 .. N) indicating the order in which subcountry names appear in the list
-on that subcountry's Wikipedia page.
+I<number> is the 3-digit number from the ISO_3166-1 page.
+
+I<sequence> is a number (1 .. N) indicating the order in which records for the same country_id
+should be accessed.
 
 See the source code of L<WWW::Scraper::Wikipedia::ISO3166::Database::Create> for details of the SQL
 used to create the tables.
 
+Lastly, in L<WWW::Scraper::Wikipedia::ISO3166::Database>, there are 4 methods for reading the 4
+tables, as well as various more general methods.
+
+=head2 A Warning about Creating the Database
+
+See also L</What is $ENV{AUTHOR_TESTING} used for?> below.
+
+If you run scripts/drop.tables.pl and scripts/create.tables.pl before running
+scripts/populate.countries.pl and scripts/populate.subcountries, then the primary keys in the
+tables will start from 1. This is good because it preempts a source of confusion.
+
+Without that step, L<SQLite|sqlite.org> will simply increment the primary keys starting from 1
+more than was previously used.
+
 =head2 What do I do if I find a mistake in the data?
 
 What data? What mistake? How do you know it's wrong?
@@ -511,21 +677,7 @@ AFAICT, sqlite3 does not have command line options, or options while running, to
 
 =back
 
-=head2 Why did you use L<Unicode::Normalize>'s NFC() for sorting?
-
-This question implies why not use NFD() instead.
-
-Run scripts/test.nfc.pl, and the output is:
-
-	code2 => AX
-	code3 => ALA
-	fc_name => åland islands
-	has_subcountries => No
-	id => 15
-	name => Åland Islands
-	timestamp => 2012-05-13 23:37:20
-
-And this (Åland Islands) is what Wikipedia displays. So, NFC() it is.
+=head2 Why did you use L<Unicode::Normalize>'s fc() for sorting?
 
 See L<http://www.perl.com/pub/2012/04>, and specifically prescription # 1.
 
@@ -541,16 +693,6 @@ writing to that copy anyway.
 
 At run-time, L<File::ShareDir> is used to find the installed version of *.sqlite.
 
-=head2 What FIPS data is included?
-
-Firstly, scripts/get.fips.pages.pl downloads some Wikipedia data, into data/List_of_FIPS_region_codes_*.html.
-
-Secondly, the latter files are parsed by scripts/populate.fips.codes.pl and the 2 reports are in
-data/wikipedia.fips.codes.txt, and data/wikipedia.fips.mismatch.log.
-
-This data is I<not> written into the SQLite database yet, but it's available in case it's included
-one day.
-
 =head1 Wikipedia's Terms of Use
 
 See L<http://wikimediafoundation.org/wiki/Terms_of_use>.
@@ -559,13 +701,17 @@ Also, since I'm distributing copies of Wikipedia-sourced material, reformatted b
 I hereby give notice that their material is released under CC-BY-SA.
 See L<http://creativecommons.org/licenses/by-sa/3.0/> for that licence.
 
+=head1 See Also
+
+L<Locale::Codes> by Sullivan Beck.
+
 =head1 References
 
 In no particular order:
 
-L<http://en.wikipedia.org/wiki/ISO_3166-2>
+L<http://en.wikipedia.org/wiki/ISO_3166-1>
 
-L<http://en.wikipedia.org/wiki/ISO_3166-1_alpha-3>
+L<http://en.wikipedia.org/wiki/ISO_3166-2>
 
 L<http://savage.net.au/Perl-modules/html/WWW/Scraper/Wikipedia/ISO3166/iso.3166-2.html>
 
