@@ -1,12 +1,9 @@
 #!/usr/bin/env perl
 
-use feature 'say';
 use strict;
-use utf8;
 use warnings;
 use warnings  qw(FATAL utf8);    # Fatalize encoding glitches.
 use open      qw(:std :utf8);    # Undeclared streams in UTF-8.
-use charnames qw(:full :short);  # Unneeded in v5.16.
 
 use Getopt::Long;
 
@@ -24,7 +21,7 @@ if ($option_parser -> getoptions
 (
 	\%option,
 	'help',
-	'verbose=s',
+	'maxlevel=s',
 	'web_page_file=s',
 ) )
 {
@@ -51,7 +48,7 @@ export.as.html.pl [options]
 
 	Options:
 	-help
-	-verbose $integer
+	-maxlevel $string
 	-web_page_file $aFileName
 
 All switches can be reduced to a single letter.
@@ -70,16 +67,11 @@ Default output: Screen.
 
 Print help and exit.
 
-=item o -verbose $integer
+=item -maxlevel => $string
 
-Print more or less progress reports. Details (more-or-less):
+Typical values: 'debug'.
 
-	0: Print nothing.
-	1: Warnings, or anything I'm working on.
-	2: The country table and specials table.
-	3: The kinds of subcountries encountered. See comments in code re 'verbose > 2'.
-
-Default: 0.
+Default: 'notice'.
 
 =item o -web_page_file $aFileName
 
