@@ -1,12 +1,9 @@
 #!/usr/bin/env perl
 
-use feature 'say';
 use strict;
-use utf8;
 use warnings;
 use warnings  qw(FATAL utf8);    # Fatalize encoding glitches.
 use open      qw(:std :utf8);    # Undeclared streams in UTF-8.
-use charnames qw(:full :short);  # Unneeded in v5.16.
 
 use Getopt::Long;
 
@@ -24,7 +21,7 @@ if ($option_parser -> getoptions
 (
 	\%option,
 	'help',
-	'verbose=s',
+	'maxlevel=s',
 ) )
 {
 	pod2usage(1) if ($option{'help'});
@@ -42,7 +39,7 @@ __END__
 
 =head1 NAME
 
-get.country.pages.pl - Get http://en.wikipedia.org/wiki/ISO_3166-2.html & the 3-letter code page
+get.country.pages.pl - Get http://en.wikipedia.org/wiki/ISO_3166-[12].html
 
 =head1 SYNOPSIS
 
@@ -50,17 +47,17 @@ get.country.pages.pl [options]
 
 	Options:
 	-help
-	-verbose $integer
+	-maxlevel $string
 
 All switches can be reduced to a single letter.
 
-Exit value: 0.
+Exit value: 0 for success.
 
-1: Input: http://en.wikipedia.org.wiki.ISO_3166-1_alpha-3
+1: Input: http://en.wikipedia.org.wiki.ISO_3166-1
 
-Output: data/en.wikipedia.org.wiki.ISO_3166-2.3.html.
+Output: data/en.wikipedia.org.wiki.ISO_3166-1.html.
 
-2: Input: http://en.wikipedia.org.wiki.ISO_3166-2.html.
+2: Input: http://en.wikipedia.org.wiki.ISO_3166-2.
 
 Output: data/en.wikipedia.org.wiki.ISO_3166-2.html.
 
@@ -72,16 +69,11 @@ Output: data/en.wikipedia.org.wiki.ISO_3166-2.html.
 
 Print help and exit.
 
-=item -verbose => $integer
+=item -maxlevel => $string
 
-Print more or less progress reports. Details (more-or-less):
+Typical values: 'debug'.
 
-	0: Print nothing.
-	1: Warnings, or anything I'm working on.
-	2: The country table and specials table.
-	3: The kinds of subcountries encountered. See comments in code re 'verbose > 2'.
-
-Default: 0.
+Default: 'notice'.
 
 =back
 
