@@ -1,12 +1,9 @@
 #!/usr/bin/env perl
 
-use feature 'say';
 use strict;
-use utf8;
 use warnings;
 use warnings  qw(FATAL utf8);    # Fatalize encoding glitches.
 use open      qw(:std :utf8);    # Undeclared streams in UTF-8.
-use charnames qw(:full :short);  # Unneeded in v5.16.
 
 use Getopt::Long;
 
@@ -25,7 +22,7 @@ if ($option_parser -> getoptions
 	\%option,
 	'code2=s',
 	'help',
-	'verbose=s',
+	'maxlevel=s',
 ) )
 {
 	pod2usage(1) if ($option{'help'});
@@ -52,7 +49,7 @@ populate.subcountry.pl [options]
 	Options:
 	-code2 $a_2_letter_country_code
 	-help
-	-verbose $integer
+	-maxlevel $string
 
 All switches can be reduced to a single letter.
 
@@ -76,16 +73,11 @@ Specify the code of the country to process.
 
 Print help and exit.
 
-=item -verbose => $integer
+=item -maxlevel => $string
 
-Print more or less progress reports. Details (more-or-less):
+Typical values: 'debug'.
 
-	0: Print nothing.
-	1: Warnings, or anything I'm working on.
-	2: The country table and specials table.
-	3: The kinds of subcountries encountered. See comments in code re 'verbose > 2'.
-
-Default: 0.
+Default: 'notice'.
 
 =back
 
