@@ -2,6 +2,8 @@
 
 use strict;
 use warnings;
+use warnings  qw(FATAL utf8);    # Fatalize encoding glitches.
+use open      qw(:std :utf8);    # Undeclared streams in UTF-8.
 
 use Getopt::Long;
 
@@ -19,7 +21,7 @@ if ($option_parser -> getoptions
 (
 	\%option,
 	'help',
-	'verbose=s',
+	'maxlevel=s',
 ) )
 {
 	pod2usage(1) if ($option{'help'});
@@ -45,7 +47,7 @@ report.statistics.pl [options]
 
 	Options:
 	-help
-	-verbose $integer
+	-maxlevel $string
 
 All switches can be reduced to a single letter.
 
@@ -63,16 +65,11 @@ Default output: Screen.
 
 Print help and exit.
 
-=item -verbose => $integer
+=item -maxlevel => $string
 
-Print more or less progress reports. Details (more-or-less):
+Typical values: 'debug'.
 
-	0: Print nothing.
-	1: Warnings, or anything I'm working on.
-	2: The country table and specials table.
-	3: The kinds of subcountries encountered. See comments in code re 'verbose > 2'.
-
-Default: 0.
+Default: 'notice'.
 
 =back
 
